"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  ApolloServerHighlightPlugin: () => ApolloServerHighlightPlugin,
  ApolloServerV3HighlightPlugin: () => ApolloServerV3HighlightPlugin,
  H: () => H,
  HIGHLIGHT_REQUEST_HEADER: () => HIGHLIGHT_REQUEST_HEADER,
  Handlers: () => handlers_exports,
  Highlight: () => Highlight
});
module.exports = __toCommonJS(src_exports);

// src/log.ts
var start = Date.now();
function log(from, ...data) {
  const prefix = `[Highlight Node.js ${(Date.now() - start) / 1e3}s] - {${from}}`;
  console.log.apply(console, [prefix, ...data]);
}

// src/sdk.ts
var HIGHLIGHT_REQUEST_HEADER = "x-highlight-request";
var _debug = false;
var highlight_obj;
var H = {
  init: (options) => {
    _debug = !!options.debug;
    try {
      highlight_obj = new Highlight(options);
    } catch (e) {
      console.warn("highlight-node init error: ", e);
    }
  },
  isInitialized: () => !!highlight_obj,
  consumeError: (error, secureSessionId, requestId) => {
    try {
      highlight_obj.consumeCustomError(error, secureSessionId, requestId);
    } catch (e) {
      console.warn("highlight-node consumeError error: ", e);
    }
  },
  consumeEvent: (secureSessionId) => {
    try {
      highlight_obj.consumeCustomEvent(secureSessionId);
    } catch (e) {
      console.warn("highlight-node consumeEvent error: ", e);
    }
  },
  recordMetric: (secureSessionId, name, value, requestId, tags) => {
    try {
      highlight_obj.recordMetric(
        secureSessionId,
        name,
        value,
        requestId,
        tags
      );
    } catch (e) {
      console.warn("highlight-node recordMetric error: ", e);
    }
  },
  flush: async () => {
    try {
      await highlight_obj.flush();
    } catch (e) {
      console.warn("highlight-node flush error: ", e);
    }
  },
  parseHeaders: (headers) => {
    try {
      if (headers && headers[HIGHLIGHT_REQUEST_HEADER]) {
        const [secureSessionId, requestId] = `${headers[HIGHLIGHT_REQUEST_HEADER]}`.split("/");
        return { secureSessionId, requestId };
      } else {
        H.log(
          `request headers do not contain ${HIGHLIGHT_REQUEST_HEADER}`
        );
      }
    } catch (e) {
      console.warn("highlight-node parseHeaders error: ", e);
    }
    return void 0;
  },
  log: (...data) => {
    if (_debug) {
      log("H", ...data);
    }
  }
};

// src/graph/generated/operations.ts
var import_graphql_tag = __toESM(require("graphql-tag"));
var MarkBackendSetupDocument = import_graphql_tag.default`
	mutation MarkBackendSetup(
		$project_id: String!
		$session_secure_id: String
	) {
		markBackendSetup(
			project_id: $project_id
			session_secure_id: $session_secure_id
		)
	}
`;
var PushMetricsDocument = import_graphql_tag.default`
	mutation PushMetrics($metrics: [MetricInput]!) {
		pushMetrics(metrics: $metrics)
	}
`;
var defaultWrapper = (action, _operationName, _operationType) => action();
function getSdk(client, withWrapper = defaultWrapper) {
  return {
    MarkBackendSetup(variables, requestHeaders) {
      return withWrapper(
        (wrappedRequestHeaders) => client.request(
          MarkBackendSetupDocument,
          variables,
          { ...requestHeaders, ...wrappedRequestHeaders }
        ),
        "MarkBackendSetup",
        "mutation"
      );
    },
    PushMetrics(variables, requestHeaders) {
      return withWrapper(
        (wrappedRequestHeaders) => client.request(
          PushMetricsDocument,
          variables,
          { ...requestHeaders, ...wrappedRequestHeaders }
        ),
        "PushMetrics",
        "mutation"
      );
    }
  };
}

// src/client.ts
var import_graphql_request = require("graphql-request");

// src/errorContext.ts
var import_lru_cache = __toESM(require("lru-cache"));
var import_fs = __toESM(require("fs"));
var CONTEXT_LINE_MAX_SIZE = 1e3;
var CONTEXT_LINES_COUNT = 5;
function coalesceSize(inputSize) {
  if (inputSize === void 0) {
    return CONTEXT_CACHE_DEFAULT_SIZE;
  } else if (inputSize <= 0) {
    return void 0;
  } else {
    return inputSize;
  }
}
var CONTEXT_CACHE_DEFAULT_SIZE = 10;
var ErrorContext = class {
  constructor(options) {
    this._sourceContextCacheSizeMB = coalesceSize(
      options.sourceContextCacheSizeMB
    );
    const lruOptions = {
      max: 1e3,
      maxSize: this._sourceContextCacheSizeMB ? this._sourceContextCacheSizeMB * 1e3 * 1e3 : void 0,
      sizeCalculation: (lines, key) => {
        return lines.size;
      }
    };
    this._cache = new import_lru_cache.default(lruOptions);
  }
  _getOrLoadLines(filename) {
    const maybeCached = this._cache.get(filename);
    if (maybeCached !== void 0) {
      return maybeCached.lines;
    }
    let size = 0;
    const contents = import_fs.default.readFileSync(filename, "utf8");
    const lineStrs = contents.split("\n");
    const lines = /* @__PURE__ */ new Map();
    lineStrs.forEach((line, idx) => {
      const curLine = line.substring(0, CONTEXT_LINE_MAX_SIZE) + "\n";
      lines.set(idx + 1, curLine);
      size += curLine.length;
    });
    this._cache.set(filename, {
      lines,
      size
    });
    return lines;
  }
  getStackFrameContext(filename, lineNumber) {
    const lines = this._getOrLoadLines(filename);
    let lineContent = lines.get(lineNumber);
    let linesBefore = "";
    for (let i = lineNumber - CONTEXT_LINES_COUNT; i < lineNumber; i++) {
      let nextLine = lines.get(i);
      if (nextLine !== void 0) {
        linesBefore += nextLine;
      }
    }
    let linesAfter = "";
    for (let i = lineNumber + 1; i < lineNumber + CONTEXT_LINES_COUNT + 1; i++) {
      let nextLine = lines.get(i);
      if (nextLine !== void 0) {
        linesAfter += nextLine;
      }
    }
    return {
      lineContent,
      linesBefore,
      linesAfter
    };
  }
};

// src/client.ts
var opentelemetry = __toESM(require("@opentelemetry/sdk-node"));
var import_auto_instrumentations_node = require("@opentelemetry/auto-instrumentations-node");
var import_exporter_trace_otlp_http = require("@opentelemetry/exporter-trace-otlp-http");
var import_api = require("@opentelemetry/api");
var OTLP_HTTP = "https://otel.highlight.io:4318";
var Highlight = class {
  constructor(options) {
    this.FLUSH_TIMEOUT = 10;
    this.BACKEND_SETUP_TIMEOUT = 15 * 60 * 1e3;
    this.metrics = [];
    this.lastBackendSetupEvent = 0;
    this._debug = !!options.debug;
    this._projectID = options.projectID;
    this._backendUrl = options.backendUrl || "https://pub.highlight.run";
    const client = new import_graphql_request.GraphQLClient(this._backendUrl, {
      headers: {}
    });
    this._graphqlSdk = getSdk(client);
    if (options.debug) {
      import_api.diag.setLogger(new import_api.DiagConsoleLogger(), import_api.DiagLogLevel.DEBUG);
    }
    this.tracer = import_api.trace.getTracer("highlight-node");
    const exporter = new import_exporter_trace_otlp_http.OTLPTraceExporter({
      url: `${options.otlpEndpoint ?? OTLP_HTTP}/v1/traces`
    });
    this.otel = new opentelemetry.NodeSDK({
      traceExporter: exporter,
      instrumentations: [(0, import_auto_instrumentations_node.getNodeAutoInstrumentations)()]
    });
    this.otel.start();
    this._intervalFunction = setInterval(
      () => this.flush(),
      this.FLUSH_TIMEOUT * 1e3
    );
    if (!options.disableErrorSourceContext) {
      this._errorContext = new ErrorContext({
        sourceContextCacheSizeMB: options.errorSourceContextCacheSizeMB
      });
    }
    this._graphqlSdk.MarkBackendSetup({
      project_id: this._projectID
    }).then(() => {
      this.lastBackendSetupEvent = Date.now();
    }).catch((e) => {
      console.warn("highlight-node error: ", e);
    });
    this._log(`Initialized SDK for project ${this._projectID}`);
  }
  _log(...data) {
    if (this._debug) {
      log("client", ...data);
    }
  }
  recordMetric(secureSessionId, name, value, requestId, tags) {
    this.metrics.push({
      session_secure_id: secureSessionId,
      group: requestId,
      name,
      value,
      category: "BACKEND",
      timestamp: new Date().toISOString(),
      tags
    });
  }
  consumeCustomError(error, secureSessionId, requestId) {
    let span = import_api.trace.getActiveSpan();
    let spanCreated = false;
    if (!span) {
      span = this.tracer.startSpan("highlight-ctx");
      spanCreated = true;
    }
    span.recordException(error);
    span.setAttributes({
      highlight_project_id: this._projectID,
      highlight_session_id: secureSessionId,
      highlight_trace_id: requestId
    });
    if (spanCreated) {
      span.end();
    }
  }
  consumeCustomEvent(secureSessionId) {
    const sendBackendSetup = Date.now() - this.lastBackendSetupEvent > this.BACKEND_SETUP_TIMEOUT;
    if (sendBackendSetup) {
      this._graphqlSdk.MarkBackendSetup({
        project_id: this._projectID,
        session_secure_id: secureSessionId
      }).then(() => {
        this.lastBackendSetupEvent = Date.now();
      }).catch((e) => {
        console.warn("highlight-node error: ", e);
      });
    }
  }
  async flushMetrics() {
    if (this.metrics.length === 0) {
      return;
    }
    const variables = {
      metrics: this.metrics
    };
    this.metrics = [];
    try {
      await this._graphqlSdk.PushMetrics(variables);
    } catch (e) {
      console.warn("highlight-node pushMetrics error: ", e);
    }
  }
  async flush() {
    await Promise.all([this.flushMetrics()]);
  }
};

// src/apollo.ts
var ApolloServerHighlightPlugin = function(options) {
  return {
    async requestDidStart(req) {
      var _a, _b, _c;
      let secureSessionId;
      let requestId;
      if ((_b = (_a = req.request.http) == null ? void 0 : _a.headers) == null ? void 0 : _b.get(HIGHLIGHT_REQUEST_HEADER)) {
        ;
        [secureSessionId, requestId] = `${(_c = req.request.http.headers) == null ? void 0 : _c.get(
          HIGHLIGHT_REQUEST_HEADER
        )}`.split("/");
      }
      H.log("processError", "extracted from headers", {
        secureSessionId,
        requestId
      });
      if (!H.isInitialized()) {
        H.init(options);
        H.log("initialized H in apollo server");
      }
      return {
        async didEncounterErrors(requestContext) {
          H.consumeEvent(secureSessionId);
          for (const error of requestContext.errors) {
            H.consumeError(error, secureSessionId, requestId);
            H.log("consumed apollo request error", error);
          }
        }
      };
    }
  };
};

// src/apolloV3.ts
var ApolloServerV3HighlightPlugin = function(options) {
  return {
    async requestDidStart(req) {
      var _a, _b, _c;
      let secureSessionId;
      let requestId;
      if ((_b = (_a = req.request.http) == null ? void 0 : _a.headers) == null ? void 0 : _b.get(HIGHLIGHT_REQUEST_HEADER)) {
        ;
        [secureSessionId, requestId] = `${(_c = req.request.http.headers) == null ? void 0 : _c.get(
          HIGHLIGHT_REQUEST_HEADER
        )}`.split("/");
      }
      H.log("processError", "extracted from headers", {
        secureSessionId,
        requestId
      });
      if (!H.isInitialized()) {
        H.init(options);
        H.log("initialized H in apollo server");
      }
      return {
        async didEncounterErrors(requestContext) {
          H.consumeEvent(secureSessionId);
          for (const error of requestContext.errors) {
            H.consumeError(error, secureSessionId, requestId);
            H.log("consumed apollo request error", error);
          }
        }
      };
    }
  };
};

// src/handlers.ts
var handlers_exports = {};
__export(handlers_exports, {
  errorHandler: () => errorHandler,
  firebaseCallableFunctionHandler: () => firebaseCallableFunctionHandler,
  firebaseHttpFunctionHandler: () => firebaseHttpFunctionHandler,
  trpcOnError: () => trpcOnError
});
function processErrorImpl(options, req, error) {
  let secureSessionId;
  let requestId;
  if (req.headers && req.headers[HIGHLIGHT_REQUEST_HEADER]) {
    ;
    [secureSessionId, requestId] = `${req.headers[HIGHLIGHT_REQUEST_HEADER]}`.split("/");
  }
  H.log("processError", "extracted from headers", {
    secureSessionId,
    requestId
  });
  if (!H.isInitialized()) {
    H.init(options);
    H.log("initialized H");
  }
  H.consumeEvent(secureSessionId);
  if (error instanceof Error) {
    H.consumeError(error, secureSessionId, requestId);
    H.log("consumed error", error);
  }
}
function errorHandler(options) {
  H.log("setting up error handler");
  return async (error, req, res, next) => {
    H.log("handling request");
    try {
      processErrorImpl(options, req, error);
    } finally {
      next(error);
    }
  };
}
async function trpcOnError({
  error,
  req
}, options) {
  try {
    if (!H.isInitialized()) {
      H.init(options);
    }
    processErrorImpl(options, req, error);
    await H.flush();
  } catch (e) {
    console.warn("highlight-node trpcOnError error:", e);
  }
}
function firebaseHttpFunctionHandler(origHandler, options) {
  return async (req, res) => {
    try {
      return await origHandler(req, res);
    } catch (e) {
      try {
        if (e instanceof Error) {
          if (!H.isInitialized()) {
            H.init(options);
          }
          processErrorImpl(options, req, e);
          await H.flush();
        }
      } catch (e2) {
        console.warn(
          "highlight-node firebaseHttpFunctionHandler error:",
          e2
        );
      }
      throw e;
    }
  };
}
function firebaseCallableFunctionHandler(origHandler, options) {
  return async (data, context) => {
    try {
      return await origHandler(data, context);
    } catch (e) {
      try {
        if (e instanceof Error) {
          if (!H.isInitialized()) {
            H.init(options);
          }
          processErrorImpl(options, context.rawRequest, e);
          await H.flush();
        }
      } catch (e2) {
        console.warn(
          "highlight-node firebaseCallableFunctionHandler error:",
          e2
        );
      }
      throw e;
    }
  };
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  ApolloServerHighlightPlugin,
  ApolloServerV3HighlightPlugin,
  H,
  HIGHLIGHT_REQUEST_HEADER,
  Handlers,
  Highlight
});
